@@ -42,6 +42,8 @@ ResourceNotFound, ResourceConflict, BulkSaveError, MultipleResultsFound
 from . import resource
 from .utils import validate_dbname
 
+from .schema.util import maybe_schema_wrapper
+
 
 DEFAULT_UUID_BATCH_COUNT = 1000
 
@@ -660,6 +662,16 @@ class Database(object):
 
         return { 'ok': False }
 
+    def raw_view(self, view_path, params):
+        if 'keys' in params:
+            keys = params.pop('keys')
+            return self.res.post(view_path, payload={ 'keys': keys }, **params)
+        else:
+            return self.res.get(view_path, **params)
+
+    def raw_temp_view(db, design, params):
+        return db.res.post('_temp_view', payload=design,
+               headers={"Content-Type": "application/json"}, **params)
 
     def view(self, view_name, schema=None, wrapper=None, **params):
         """ get view results from database. viewname is generally
@@ -678,35 +690,6 @@ class Database(object):
         @param params: params of the view
 
         """
-        def get_multi_wrapper(classes, wrap_doc=True,
-                dynamic_properties=True):
-            def wrapper(row):
-                data = row.get('value')
-                docid = row.get('id')
-                doc = row.get('doc')
-                if doc is not None and wrap_doc:
-                    try:
-                        cls = classes[doc.get('doc_type')]
-                        cls._allow_dynamic_properties = dynamic_properties
-                        return cls.wrap(doc)
-                    except KeyError:
-                        return row
-                elif not data or data is None:
-                    return row
-                elif not isinstance(data, dict) or not docid:
-                    return row
-                else:
-                    try:
-                        cls = classes[data.get('doc_type')]
-                        data['_id'] = docid
-                        if 'rev' in data:
-                            data['_rev'] = data.pop('rev')
-                        cls._allow_dynamic_properties = dynamic_properties
-                        return cls.wrap(data)
-                    except KeyError:
-                        return row
-
-            return wrapper
 
         if view_name.startswith('/'):
             view_name = view_name[1:]
@@ -720,47 +703,29 @@ class Database(object):
             vname = '/'.join(view_name)
             view_path = '_design/%s/_view/%s' % (dname, vname)
 
-        if schema is not None:
-            wrap_doc = params.get('wrap_doc', True)
-            dynamic_properties = params.get('dynamic_properties', True)
-
-            if hasattr(schema, "wrap"):
-                if hasattr(schema, '_doc_type'):
-                    schema = {schema._doc_type: schema}
-                    wrapper = get_multi_wrapper(schema, wrap_doc=wrap_doc,
-                            dynamic_properties=dynamic_properties)
-                else:
-                    wrapper = schema.wrap
-            else:
-                if isinstance(schema, list):
-                    schema = dict([(s._doc_type, s) for s in schema])
-
-                wrapper = get_multi_wrapper(schema, wrap_doc=wrap_doc,
-                    dynamic_properties=dynamic_properties)
-
-        return View(self, view_path, wrapper=wrapper)(**params)
+        return ViewResults(self.raw_view, view_path, wrapper, schema, params)
 
     def temp_view(self, design, schema=None, wrapper=None, **params):
         """ get adhoc view results. Like view it reeturn a ViewResult object."""
-        if schema is not None:
-            if not hasattr(schema, 'wrap'):
-                raise AttributeError("no 'wrap' method found in obj %s)" % str(schema))
-            wrapper = schema.wrap
-        return TempView(self, design, wrapper=wrapper)(**params)
+        return ViewResults(self.raw_temp_view, design, wrapper, schema, params)
 
-    def search( self, view_name, handler='_fti/_design', wrapper=None, **params):
+    def search( self, view_name, handler='_fti/_design', wrapper=None, schema=None, **params):
         """ Search. Return results from search. Use couchdb-lucene
         with its default settings by default."""
-        return View(self, "/%s/%s" % (handler, view_name), wrapper=wrapper)(**params)
+        return ViewResults(self, self.raw_view,
+                    "/%s/%s" % (handler, view_name), 
+                    wrapper=wrapper, schema=schema, params=params)
 
     def documents(self, schema=None, wrapper=None, **params):
         """ return a ViewResults objects containing all documents.
         This is a shorthand to view function.
         """
-        return View(self, '_all_docs', schema=schema,
-                wrapper=wrapper)(**params)
+        return ViewResults(self.raw_view, '_all_docs',
+                wrapper=wrapper, schema=schema, params=params)
     iterdocuments = documents
 
+
+
     def put_attachment(self, doc, content, name=None, content_type=None,
             content_length=None, headers=None):
         """ Add attachement to a document. All attachments are streamed.
@@ -907,7 +872,7 @@ class ViewResults(object):
     Object to retrieve view results.
     """
 
-    def __init__(self, view, **params):
+    def __init__(self, fetch, arg, wrapper, schema, params):
         """
         Constructor of ViewResults object
 
@@ -915,8 +880,33 @@ class ViewResults(object):
         @param params: params to apply when fetching view.
 
         """
-        self.view = view
-        self.params = params
+        assert not (wrapper and schema)
+        wrap_doc = params.get('wrap_doc', schema is not None)
+        if schema:
+            schema = maybe_schema_wrapper(None, schema, params)
+            def row_wrapper(row):
+                data = row.get('value')
+                docid = row.get('id')
+                doc = row.get('doc')
+                if doc is not None and wrap_doc:
+                    return schema(doc)
+                elif not data or data is None:
+                    return row
+                elif not isinstance(data, dict) or not docid:
+                    return row
+                else:
+                    data['_id'] = docid
+                    if 'rev' in data:
+                        data['_rev'] = data.pop('rev')
+                    return schema(data)
+        else:
+            def row_wrapper(row):
+                return row
+
+        self._fetch = fetch
+        self._arg = arg
+        self.wrapper = wrapper or row_wrapper
+        self.params = params or {}
         self._result_cache = None
         self._total_rows = None
         self._offset = 0
@@ -925,12 +915,9 @@ class ViewResults(object):
     def iterator(self):
         self._fetch_if_needed()
         rows = self._result_cache.get('rows', [])
-        wrapper = self.view._wrapper
+        wrapper = self.wrapper
         for row in rows:
-            if  wrapper is not None:
-                yield self.view._wrapper(row)
-            else:
-                yield row
+            yield wrapper(row)
 
     def first(self):
         """
@@ -984,7 +971,7 @@ class ViewResults(object):
                 pass
         self._dynamic_keys = []
 
-        self._result_cache = self.view._exec(**self.params).json_body
+        self._result_cache = self.fetch_raw().json_body
         self._total_rows = self._result_cache.get('total_rows')
         self._offset = self._result_cache.get('offset', 0)
 
@@ -998,7 +985,7 @@ class ViewResults(object):
 
     def fetch_raw(self):
         """ retrive the raw result """
-        return self.view._exec(**self.params)
+        return self._fetch(self._arg, self.params)
 
     def _fetch_if_needed(self):
         if not self._result_cache:
@@ -1031,7 +1018,7 @@ class ViewResults(object):
         else:
             params['key'] = key
 
-        return ViewResults(self.view, **params)
+        return ViewResults(self._fetch, self._arg, wrapper=wrapper, params=params)
 
     def __iter__(self):
         return self.iterator()
@@ -1043,44 +1030,4 @@ class ViewResults(object):
         return bool(len(self))
 
 
-class ViewInterface(object):
-    """ Generic object interface used by View and TempView objects. """
 
-    def __init__(self, db, wrapper=None):
-        self._db = db
-        self._wrapper = wrapper
-
-    def __call__(self, **params):
-        return ViewResults(self, **params)
-
-    def __iter__(self):
-        return self()
-
-    def _exec(self, **params):
-        raise NotImplementedError
-
-class View(ViewInterface):
-    """ Object used to wrap a view and return ViewResults.
-    Generally called via the `view` method in a `Database` instance. """
-
-    def __init__(self, db, view_path, wrapper=None):
-        ViewInterface.__init__(self, db, wrapper=wrapper)
-        self.view_path = view_path
-
-    def _exec(self, **params):
-        if 'keys' in params:
-            keys = params.pop('keys')
-            return self._db.res.post(self.view_path, payload={ 'keys': keys }, **params)
-        else:
-            return self._db.res.get(self.view_path, **params)
-
-class TempView(ViewInterface):
-    """ Object used to wrap a temporary and return ViewResults. """
-    def __init__(self, db, design, wrapper=None):
-        ViewInterface.__init__(self, db, wrapper=wrapper)
-        self.design = design
-        self._wrapper = wrapper
-
-    def _exec(self, **params):
-        return self._db.res.post('_temp_view', payload=self.design,
-            headers={"Content-Type": "application/json"}, **params)
