@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -
 #
-# This file is part of couchdbkit released under the MIT license. 
+# This file is part of couchdbkit released under the MIT license.
 # See the NOTICE for more information.
 
 """ properties used by Document object """
@@ -15,23 +15,25 @@ try:
 
     def is_iterable(c):
         return isinstance(c, Iterable)
-except ImportError:
-    from sets import Set as MutableSet
-    
-    def is_iterable(o):
-        return hasattr(c, '__iter__')
 
-from couchdbkit.exceptions import BadValueError 
+    support_setproperty = True
+except ImportError:
+    support_setproperty = False
+
+from couchdbkit.exceptions import BadValueError
 
 __all__ = ['ALLOWED_PROPERTY_TYPES', 'Property', 'StringProperty',
         'IntegerProperty', 'DecimalProperty', 'BooleanProperty',
         'FloatProperty', 'DateTimeProperty', 'DateProperty',
         'TimeProperty', 'DictProperty', 'ListProperty',
-        'StringListProperty', 'SetProperty',
+        'StringListProperty',
         'dict_to_json', 'list_to_json',
         'value_to_json', 'MAP_TYPES_PROPERTIES', 'value_to_python',
         'dict_to_python', 'list_to_python', 'convert_property',
-        'value_to_property', 'LazyDict', 'LazyList', 'LazySet']
+        'value_to_property', 'LazyDict', 'LazyList']
+
+if support_setproperty:
+    __all__ += ['SetProperty', 'LasySet']
 
 ALLOWED_PROPERTY_TYPES = set([
     basestring,
@@ -64,14 +66,14 @@ class Property(object):
     def __init__(self, verbose_name=None, name=None,
             default=None, required=False, validators=None,
             choices=None):
-        """ Default constructor for a property. 
+        """ Default constructor for a property.
 
         :param verbose_name: str, verbose name of field, could
                 be use for description
         :param name: str, name of field
         :param default: default value
         :param required: True if field is required, default is False
-        :param validators: list of callable or callable, field validators 
+        :param validators: list of callable or callable, field validators
         function that are executed when document is saved.
         """
         self.verbose_name = verbose_name
@@ -168,8 +170,8 @@ class Property(object):
     data_type = None
 
 class StringProperty(Property):
-    """ string property str or unicode property 
-    
+    """ string property str or unicode property
+
     *Value type*: unicode
     """
 
@@ -190,8 +192,8 @@ class StringProperty(Property):
     data_type = unicode
 
 class IntegerProperty(Property):
-    """ Integer property. map to int 
-    
+    """ Integer property. map to int
+
     *Value type*: int
     """
     to_python = int
@@ -218,8 +220,8 @@ class IntegerProperty(Property):
 LongProperty = IntegerProperty
 
 class FloatProperty(Property):
-    """ Float property, map to python float 
-    
+    """ Float property, map to python float
+
     *Value type*: float
     """
     to_python = float
@@ -242,7 +244,7 @@ Number = FloatProperty
 
 class BooleanProperty(Property):
     """ Boolean property, map to python bool
-    
+
     *ValueType*: bool
     """
     to_python = bool
@@ -268,7 +270,7 @@ class BooleanProperty(Property):
 
 class DecimalProperty(Property):
     """ Decimal property, map to Decimal python object
-    
+
     *ValueType*: decimal.Decimal
     """
     data_type = decimal.Decimal
@@ -283,7 +285,7 @@ class DateTimeProperty(Property):
     """DateTime property. It convert iso3339 string
     to python and vice-versa. Map to datetime.datetime
     object.
-    
+
     *ValueType*: datetime.datetime
     """
 
@@ -337,7 +339,7 @@ class DateTimeProperty(Property):
 class DateProperty(DateTimeProperty):
     """ Date property, like DateTime property but only
     for Date. Map to datetime.date object
-    
+
     *ValueType*: datetime.date
     """
     data_type = datetime.date
@@ -363,7 +365,7 @@ class DateProperty(DateTimeProperty):
 class TimeProperty(DateTimeProperty):
     """ Date property, like DateTime property but only
     for time. Map to datetime.time object
-    
+
     *ValueType*: datetime.time
     """
 
@@ -458,12 +460,12 @@ class ListProperty(Property):
             required=False, item_type=None, **kwds):
         """Construct ListProperty.
 
-    
+
          :args verbose_name: Optional verbose name.
          :args default: Optional default value; if omitted, an empty list is used.
          :args**kwds: Optional additional keyword arguments, passed to base class.
 
-        
+
         """
         if default is None:
             default = []
@@ -526,95 +528,19 @@ class StringListProperty(ListProperty):
             default=default, required=required, item_type=basestring, **kwds)
 
 
-class SetProperty(Property):
-    """A property that stores a Python set as a list of unique
-    elements.
-
-    Note that Python set operations like union that return a set
-    object do not alter list that will be stored with the next save,
-    while operations like update that change a set object in-place do
-    keep the list in sync.
-    """
-    def __init__(self, verbose_name=None, default=None, required=None,
-                 item_type=None, **kwds):
-        """Construct SetProperty.
-
-         :args verbose_name: Optional verbose name.
-
-         :args default: Optional default value; if omitted, an empty
-                        set is used.
-
-         :args required: True if field is required, default is False.
-
-         :args item_type: Optional data type of items that set
-                          contains.  Used to assist with JSON
-                          serialization/deserialization when data is
-                          stored/retireved.
-
-         :args **kwds: Optional additional keyword arguments, passed to
-                       base class.
-         """
-        if default is None:
-            default = set()
-        if item_type is not None and item_type not in ALLOWED_PROPERTY_TYPES:
-            raise ValueError('item_type %s not in %s'
-                             % (item_type, ALLOWED_PROPERTY_TYPES))
-        self.item_type = item_type
-        super(SetProperty, self).__init__(
-            verbose_name=verbose_name, default=default, required=required,
-            **kwds)
-
-    data_type = set
-
-    def validate(self, value, required=True):
-        value = super(SetProperty, self).validate(value, required=required)
-        if value and value is not None:
-            if not isinstance(value, MutableSet):
-                raise BadValueError('Property %s must be a set' % self.name)
-            value = self.validate_set_contents(value)
-        return value
-
-    def validate_set_contents(self, value):
-        try:
-            value = validate_set_content(value, item_type=self.item_type)
-        except BadValueError:
-            raise BadValueError(
-                'Items of %s set must all be in %s' %
-                    (self.name, ALLOWED_PROPERTY_TYPES))
-        return value
-
-    def default_value(self):
-        """Return default value for set.
-
-        Because the property supplied to 'default' is a static value,
-        that value must be shallow copied to prevent all fields with
-        default values from sharing the same instance.
-
-        Returns:
-          Copy of the default value.
-        """
-        value = super(SetProperty, self).default_value()
-        if value is None:
-            return set()
-        return value.copy()
-
-    def to_python(self, value):
-        return LazySet(value, item_type=self.item_type)
-
-    def to_json(self, value):
-        return value_to_json(value, item_type=self.item_type)
 
 
-# structures proxy
+
+#  dict proxy
 
 class LazyDict(dict):
-    """ object to make sure we keep updated of dict 
+    """ object to make sure we keep updated of dict
     in _doc. We just override a dict and maintain change in
     doc reference (doc[keyt] obviously).
-    
+
     if init_vals is specified, doc is overwritten
-    with the dict given. Otherwise, the values already in 
-    doc are used. 
+    with the dict given. Otherwise, the values already in
+    doc are used.
     """
 
     def __init__(self, doc, item_type=None, init_vals=None):
@@ -681,14 +607,15 @@ class LazyDict(dict):
         self.doc.clear()
         super(LazyDict, self).clear()
 
+
 class LazyList(list):
-    """ object to make sure we keep update of list 
+    """ object to make sure we keep update of list
     in _doc. We just override a list and maintain change in
     doc reference (doc[index] obviously).
 
     if init_vals is specified, doc is overwritten
-    with the list given. Otherwise, the values already in 
-    doc are used. 
+    with the list given. Otherwise, the values already in
+    doc are used.
     """
 
     def __init__(self, doc, item_type=None, init_vals=None):
@@ -795,130 +722,209 @@ class LazyList(list):
         self.doc.reverse()
         list.reverse(self)
 
+if support_setproperty:
+    class SetProperty(Property):
+        """A property that stores a Python set as a list of unique
+        elements.
 
-class LazySet(MutableSet):
-    """Object to make sure that we keep set and _doc synchronized.
+        Note that Python set operations like union that return a set
+        object do not alter list that will be stored with the next save,
+        while operations like update that change a set object in-place do
+        keep the list in sync.
+        """
+        def __init__(self, verbose_name=None, default=None, required=None,
+                     item_type=None, **kwds):
+            """Construct SetProperty.
 
-    We sub-class MutableSet and maintain changes in doc.
+             :args verbose_name: Optional verbose name.
 
-    Note that methods like union that return a set object do not
-    alter _doc, while methods like update that change a set object
-    in-place do keep _doc in sync.
-    """
-    def _map_named_operation(opname):
-        fn = getattr(MutableSet, opname)
-        if hasattr(fn, 'im_func'):
-            fn = fn.im_func
-        def method(self, other, fn=fn):
+             :args default: Optional default value; if omitted, an empty
+                            set is used.
+
+             :args required: True if field is required, default is False.
+
+             :args item_type: Optional data type of items that set
+                              contains.  Used to assist with JSON
+                              serialization/deserialization when data is
+                              stored/retireved.
+
+             :args **kwds: Optional additional keyword arguments, passed to
+                           base class.
+             """
+            if default is None:
+                default = set()
+            if item_type is not None and item_type not in ALLOWED_PROPERTY_TYPES:
+                raise ValueError('item_type %s not in %s'
+                                 % (item_type, ALLOWED_PROPERTY_TYPES))
+            self.item_type = item_type
+            super(SetProperty, self).__init__(
+                verbose_name=verbose_name, default=default, required=required,
+                **kwds)
+
+        data_type = set
+
+        def validate(self, value, required=True):
+            value = super(SetProperty, self).validate(value, required=required)
+            if value and value is not None:
+                if not isinstance(value, MutableSet):
+                    raise BadValueError('Property %s must be a set' % self.name)
+                value = self.validate_set_contents(value)
+            return value
+
+        def validate_set_contents(self, value):
+            try:
+                value = validate_set_content(value, item_type=self.item_type)
+            except BadValueError:
+                raise BadValueError(
+                    'Items of %s set must all be in %s' %
+                        (self.name, ALLOWED_PROPERTY_TYPES))
+            return value
+
+        def default_value(self):
+            """Return default value for set.
+
+            Because the property supplied to 'default' is a static value,
+            that value must be shallow copied to prevent all fields with
+            default values from sharing the same instance.
+
+            Returns:
+              Copy of the default value.
+            """
+            value = super(SetProperty, self).default_value()
+            if value is None:
+                return set()
+            return value.copy()
+
+        def to_python(self, value):
+            return LazySet(value, item_type=self.item_type)
+
+        def to_json(self, value):
+            return value_to_json(value, item_type=self.item_type)
+
+
+    class LazySet(MutableSet):
+        """Object to make sure that we keep set and _doc synchronized.
+
+        We sub-class MutableSet and maintain changes in doc.
+
+        Note that methods like union that return a set object do not
+        alter _doc, while methods like update that change a set object
+        in-place do keep _doc in sync.
+        """
+        def _map_named_operation(opname):
+            fn = getattr(MutableSet, opname)
+            if hasattr(fn, 'im_func'):
+                fn = fn.im_func
+            def method(self, other, fn=fn):
+                if not isinstance(other, MutableSet):
+                    other = self._from_iterable(other)
+                return fn(self, other)
+            return method
+
+        issubset = _map_named_operation('__le__')
+        issuperset = _map_named_operation('__ge__')
+        symmetric_difference = _map_named_operation('__xor__')
+
+        def __init__(self, doc, item_type=None):
+            self.item_type = item_type
+            self.doc = doc
+            self.elements = set(value_to_python(value, self.item_type)
+                                for value in self.doc)
+
+        def __repr__(self):
+            return '%s(%r)' % (type(self).__name__, list(self))
+
+        @classmethod
+        def _from_iterable(cls, it):
+            return cls(it)
+
+        def __iand__(self, iterator):
+            for value in (self.elements - iterator):
+                self.elements.discard(value)
+            return self
+
+        def __iter__(self):
+            return iter(element for element in self.elements)
+
+        def __len__(self):
+            return len(self.elements)
+
+        def __contains__(self, item):
+            return item in self.elements
+
+        def __xor__(self, other):
             if not isinstance(other, MutableSet):
+                if not is_iterable(other):
+                    return NotImplemented
                 other = self._from_iterable(other)
-            return fn(self, other)
-        return method
+            return (self.elements - other) | (other - self.elements)
 
-    issubset = _map_named_operation('__le__')
-    issuperset = _map_named_operation('__ge__')
-    symmetric_difference = _map_named_operation('__xor__')
-
-    def __init__(self, doc, item_type=None):
-        self.item_type = item_type
-        self.doc = doc
-        self.elements = set(value_to_python(value, self.item_type)
-                            for value in self.doc)
-
-    def __repr__(self):
-        return '%s(%r)' % (type(self).__name__, list(self))
-
-    @classmethod
-    def _from_iterable(cls, it):
-        return cls(it)
-
-    def __iand__(self, iterator):
-        for value in (self.elements - iterator):
-            self.elements.discard(value)
-        return self
-
-    def __iter__(self):
-        return iter(element for element in self.elements)
-
-    def __len__(self):
-        return len(self.elements)
-
-    def __contains__(self, item):
-        return item in self.elements
-
-    def __xor__(self, other):
-        if not isinstance(other, MutableSet):
-            if not is_iterable(Other):
+        def __gt__(self, other):
+            if not isinstance(other, MutableSet):
                 return NotImplemented
-            other = self._from_iterable(other)
-        return (self.elements - other) | (other - self.elements)
+            return other < self.elements
 
-    def __gt__(self, other):
-        if not isinstance(other, MutableSet):
-            return NotImplemented
-        return other < self.elements
+        def __ge__(self, other):
+            if not isinstance(other, MutableSet):
+                return NotImplemented
+            return other <= self.elements
 
-    def __ge__(self, other):
-        if not isinstance(other, MutableSet):
-            return NotImplemented
-        return other <= self.elements
+        def __ne__(self, other):
+            return not (self.elements == other)
 
-    def __ne__(self, other):
-        return not (self.elements == other)
+        def add(self, value):
+            self.elements.add(value)
+            if value not in self.doc:
+                self.doc.append(value_to_json(value, item_type=self.item_type))
 
-    def add(self, value):
-        self.elements.add(value)
-        if value not in self.doc:
-            self.doc.append(value_to_json(value, item_type=self.item_type))
+        def copy(self):
+            return self.elements.copy()
 
-    def copy(self):
-        return self.elements.copy()
+        def difference(self, other, *args):
+            return self.elements.difference(other, *args)
 
-    def difference(self, other, *args):
-        return self.elements.difference(other, *args)
-
-    def difference_update(self, other, *args):
-        for value in other:
-            self.discard(value)
-        for arg in args:
-            self.difference_update(arg)
-
-    def discard(self, value):
-        self.elements.discard(value)
-        try:
-            self.doc.remove(value)
-        except ValueError:
-            pass
-
-    def intersection(self, other, *args):
-        return self.elements.intersection(other, *args)
-
-    def intersection_update(self, other, *args):
-        if not isinstance(other, MutableSet):
-            other = set(other)
-        for value in self.elements - other:
-            self.discard(value)
-        for arg in args:
-            self.intersection_update(arg)
-
-    def symmetric_difference_update(self, other):
-        if not isinstance(other, MutableSet):
-            other = set(other)
-        for value in other:
-            if value in self.elements:
+        def difference_update(self, other, *args):
+            for value in other:
                 self.discard(value)
-            else:
-                self.add(value)
+            for arg in args:
+                self.difference_update(arg)
 
-    def union(self, other, *args):
-        return self.elements.union(other, *args)
+        def discard(self, value):
+            self.elements.discard(value)
+            try:
+                self.doc.remove(value)
+            except ValueError:
+                pass
 
-    def update(self, other, *args):
-        self.elements.update(other, *args)
-        for element in self.elements:
-            if element not in self.doc:
-                self.doc.append(
-                    value_to_json(element, item_type=self.item_type))
+        def intersection(self, other, *args):
+            return self.elements.intersection(other, *args)
+
+        def intersection_update(self, other, *args):
+            if not isinstance(other, MutableSet):
+                other = set(other)
+            for value in self.elements - other:
+                self.discard(value)
+            for arg in args:
+                self.intersection_update(arg)
+
+        def symmetric_difference_update(self, other):
+            if not isinstance(other, MutableSet):
+                other = set(other)
+            for value in other:
+                if value in self.elements:
+                    self.discard(value)
+                else:
+                    self.add(value)
+
+        def union(self, other, *args):
+            return self.elements.union(other, *args)
+
+        def update(self, other, *args):
+            self.elements.update(other, *args)
+            for element in self.elements:
+                if element not in self.doc:
+                    self.doc.append(
+                        value_to_json(element, item_type=self.item_type))
 
 # some mapping
 
@@ -934,10 +940,12 @@ MAP_TYPES_PROPERTIES = {
         long: LongProperty,
         float: FloatProperty,
         list: ListProperty,
-        dict: DictProperty,
-        set: SetProperty,
+        dict: DictProperty
 }
 
+if support_setproperty:
+    MAP_TYPES_PROPERTIES[set] = SetProperty
+
 def convert_property(value):
     """ convert a value to json from Property._to_json """
     if type(value) in MAP_TYPES_PROPERTIES:
@@ -995,7 +1003,7 @@ def list_to_json(value, item_type=None):
 def value_to_json(value, item_type=None):
     """ convert a value to json using appropriate regexp.
     For Dates we use ISO 8601. Decimal are converted to string.
-    
+
     """
     if isinstance(value, datetime.datetime) and is_type_ok(item_type, datetime.datetime):
         value = value.replace(microsecond=0).isoformat() + 'Z'
